use crate::ast::{FullyQualifiedName, Identifier};
use crate::name_analysis::symbol::Symbol;
use std::borrow::Cow;
use std::range::Range;

pub trait Named {
    fn name(&self) -> Cow<'_, str>;
    
    fn file_id(&self) -> usize;
    fn range(&self) -> Range<usize>;

    fn set_scope_id(&mut self, scope_id: usize);
    fn scope_id(&self) -> Option<usize>;

    fn set_saved_symbol(&mut self, symbol: Symbol);
    fn saved_symbol(&self) -> Option<Symbol>;
}

impl Named for Identifier {
    fn name(&self) -> Cow<'_, str> {
        Cow::Borrowed(&self.name)
    }
    
    fn file_id(&self) -> usize {
        self.file_id
    }
    
    fn range(&self) -> Range<usize> {
        self.range
    }

    fn set_scope_id(&mut self, id: usize) {
        self.scope_id = Some(id);
    }

    fn scope_id(&self) -> Option<usize> {
        self.scope_id
    }

    fn set_saved_symbol(&mut self, saved_symbol: Symbol) {
        self.saved_symbol = Some(saved_symbol);
    }

    fn saved_symbol(&self) -> Option<Symbol> {
        self.saved_symbol.clone()
    }
}

impl Named for FullyQualifiedName {
    fn name(&self) -> Cow<'_, str> {
        if self.identifiers.len() == 1 {
            self.identifiers[0].name()
        } else {
            let mut acc = String::new();
            for (i, identifier) in self.identifiers.iter().enumerate() {
                acc += &identifier.name();
                if i < self.identifiers.len() - 1 {
                    acc += "::";
                }
            }
            Cow::Owned(acc)
        }
    }
    
    fn file_id(&self) -> usize {
        self.file_id
    }
    
    fn range(&self) -> Range<usize> {
        self.range
    }

    fn set_scope_id(&mut self, id: usize) {
        self.scope_id = Some(id);
    }

    fn scope_id(&self) -> Option<usize> {
        self.scope_id
    }

    fn set_saved_symbol(&mut self, symbol: Symbol) {
        self.saved_symbol = Some(symbol);
    }

    fn saved_symbol(&self) -> Option<Symbol> {
        self.saved_symbol.clone()
    }
}
