use deimos::ast::build::build_ast;
use deimos::ast::unparse::Unparse;
use deimos::parser::{DeimosParser, Rule};
use deimos::util::indent_writer::IndentWriter;
use pest::Parser;
use std::path::PathBuf;

pub fn unparse(path: &PathBuf) {
    let src = std::fs::read_to_string(path).expect(&format!("Could not read {:?}", path));
    let parse_result = DeimosParser::parse(Rule::CompilationUnit, &src);
    match parse_result {
        Ok(mut pairs) => {
            let compilation_unit_pair = pairs.next().unwrap();
            let compilation_unit = build_ast(&path.display().to_string(), 0, compilation_unit_pair);
            let mut writer = IndentWriter::new(0, "  ", Box::new(std::io::stdout()));
            compilation_unit
                .unparse(&mut writer)
                .expect("Failed to write to string.");
        }
        Err(e) => {
            eprintln!("{}", e);
        }
    }
}
